/*	Preferences.h	*/
/*	$Id: Preferences.h,v 1.1 1997/06/06 18:37:03 hplus Exp hplus $	*/
/*	Simple C++ wrapper for the libprefs C-level API	*/
/*	This C++ header is not extensively tested.			*/
/*	Please verify that it works as you want it to.		*/

#if !defined(PREFERENCES_H)
#define PREFERENCES_H

#pragma once

#if !defined(LIBPREFS_H)
#include "libprefs.h"
#endif


class Preferences
{
	friend class PreferenceSet;
				/*	not implemented!	*/
								Preferences(
									const Preferences &		dummy);
		Preferences &			operator=(
									const Preferences &		dummy);
public:
								Preferences(
									const char *			application)
									{
										fHandle = NULL;
										fInit = PREFInit(application, &fHandle);
									}
								~Preferences()
									{
										if (fHandle != NULL)
											PREFShutdown(fHandle);
									}

		status_t				InitCheck()
									{
										return fInit;
									}

								operator PREFHandle()
									{
										return fHandle;
									}
protected:

		status_t				fInit;
		PREFHandle				fHandle;
};


class PreferenceSet
{
					/*	not implemented on purpose!	*/
								PreferenceSet(
									const PreferenceSet &	dummy);
		PreferenceSet &			operator=(
									const PreferenceSet &	dummy);

public:
					/*	user-level means per-user settings	*/
								PreferenceSet(
									Preferences &			preferences,
									const char *			set_name,
									bool					user_level) :
									fPreferences(preferences)
									{
										fData = NULL;
										fInit = PREFLoadSet(preferences.fHandle, set_name, user_level, &fData);
									}
								~PreferenceSet()
									{
										if (fData != NULL)
											PREFDisposeSet(&fData);
									}

		status_t				InitCheck()
									{
										return fInit;
									}

					/*	save is NOT automatically done	*/
		status_t				Save()
									{
										return PREFSaveSet(fData);
									}
		status_t				Reload()
									{
										return PREFReloadSet(fData);
									}

		status_t				SetData(
									const char *			name,
									const void *			data,
									size_t					size,
									uint32					type = B_RAW_TYPE)
									{
										return PREFSetData(fData, name, data, size, type);
									}

		status_t				Delete(
									const char *			name)
									{
										return PREFDelete(fData, name);
									}

					/*	pointer is pointer to original	*/
		status_t				GetData(
									const char *			name,
									const void *&			out_data,
									ssize_t &				out_size,
									uint32 &				out_type)
									{
										return PREFGetData(fData, name, &out_data, &out_size, &out_type);
									}

								operator PREFData()
									{
										return fData;
									}

/*	Useful format getters and setters - you can add as many as you want.	*/
/*	The major ones missing are the other int flavours, float flavours,	*/
/*	and messages. These will be added sometime later.						*/
/*	Please note that an entry_ref is NOT something you want to save in	*/
/*	a preference object, since the IDs used therein may change across		*/
/*	reboots. Save a path, or, preferrably, an alias.						*/
		status_t				GetPoint(
									const char *			name,
									BPoint &				value)
									{
										const void * data;
										ssize_t size;
										uint32 type;
										status_t err = GetData(name, data, size, type);
										if (err >= B_OK)
										{
											if ((type != B_POINT_TYPE) || (size != sizeof(BPoint)))
												err = B_MISMATCHED_VALUES;
											else
												memcpy(&value, data, size);
										}
										return err;
									}
		status_t				SetPoint(
									const char *			name,
									const BPoint &			value)
									{
										return SetData(name, &value, sizeof(value), B_POINT_TYPE);
									}
		status_t				GetRect(
									const char *			name,
									BRect &					rect)
									{
										const void * data;
										ssize_t size;
										uint32 type;
										status_t err = GetData(name, data, size, type);
										if (err >= B_OK)
										{
											if ((type != B_RECT_TYPE) || (size != sizeof(BRect)))
												err = B_MISMATCHED_VALUES;
											else
												memcpy(&rect, data, sizeof(BRect));
										}
										return err;
									}
		status_t				SetRect(
									const char *			name,
									const BRect &			value)
									{
										return SetData(name, &value, sizeof(value), B_RECT_TYPE);
									}
		status_t				GetInt32(
									const char *			name,
									int32 &					value)
									{
										const void * data;
										ssize_t size;
										uint32 type;
										status_t err = GetData(name, data, size, type);
										if (err >= B_OK)
										{
											if ((type != B_INT32_TYPE) || (size != sizeof(int32)))
												err = B_MISMATCHED_VALUES;
											else
												memcpy(&value, data, size);
										}
										return err;
									}
		status_t				SetInt32(
									const char *			name,
									const int32 &			value)
									{
										return SetData(name, &value, sizeof(value), B_INT32_TYPE);
									}
		status_t				GetColor(
									const char *			name,
									rgb_color &				value)
									{
										const void * data;
										ssize_t size;
										uint32 type;
										status_t err = GetData(name, data, size, type);
										if (err >= B_OK)
										{
											if ((type != B_RGB_COLOR_TYPE) || (size != sizeof(value)))
												err = B_MISMATCHED_VALUES;
											else
												memcpy(&value, data, size);
										}
										return err;
									}
		status_t				SetColor(
									const char *			name,
									const rgb_color &		value)
									{
										return SetData(name, &value, sizeof(value), B_RGB_COLOR_TYPE);
									}
		status_t				GetString(
									const char *			name,
									char *					buffer,
									size_t					bufsize)
									{
										const void * data;
										ssize_t size;
										uint32 type;
										status_t err = GetData(name, data, size, type);
										if (err >= B_OK)
										{
											if ((type != B_STRING_TYPE) || (size > bufsize))
												err = B_MISMATCHED_VALUES;
											else
												memcpy(buffer, data, size);
										}
										return err;
									}
		status_t				SetString(
									const char *			name,
									const char *			value)
									{
										return SetData(name, value, strlen(value)+1, B_STRING_TYPE);
									}
protected:

		Preferences &			fPreferences;
		status_t				fInit;
		PREFData				fData;
};


class PrefTransaction
{
						/*	unimplemented on purpose!	*/
								PrefTransaction(
									const PrefTransaction &	trans);
		PrefTransaction &		operator=(
									const PrefTransaction &	trans);
public:

								PrefTransaction(
									PreferenceSet &			set)
									{
										fInit = PREFBeginTransaction(set, &fTransaction);
									}
								~PrefTransaction()
									{
										if (fInit >= B_OK)
											PREFEndTransaction(fTransaction);
									}

		status_t				InitCheck()
									{
										return fInit;
									}

protected:

		status_t				fInit;
		PREFTransaction			fTransaction;
};


#endif	/*	PREFERNECES_H		*/


